#include "RIPCTest.h"


/**
 * Test for shared and exclusive locks
 * Usage: start N processes by "Bank N 0" command
 * Output: none
 */
class Bank : public RIPCTest { 
  public:
    class Account { 
      public:
	long amount;
	Account(long value) { 
	    amount = value;
	}
    };

    void run(RIPCSession* session, int id) 
    { 
	long initialAmount = nIterations*nProcesses;
	Account* srcAcc = new Account(initialAmount);
	Account* dstAcc = new Account(0);	
	RIPCSharedMemory* srcShm = session->createSharedMemory("src", srcAcc, sizeof(srcAcc));
	RIPCSharedMemory* dstShm = session->createSharedMemory("dst", dstAcc, sizeof(dstAcc));
	RIPCLock* srcLock = session->createLock("src");
	RIPCLock* dstLock = session->createLock("dst");
	RIPCBarrier* barrier = session->createBarrier("bank", nProcesses);

	for (int i = 1; i <= nIterations; i++) { 
	    srcLock->sharedLock();
	    dstLock->sharedLock();
	    srcAcc = (Account*)srcShm->get();
	    dstAcc = (Account*)dstShm->get();
	    assert(srcAcc->amount + dstAcc->amount == initialAmount);
	    srcLock->unlock();
	    dstLock->unlock();
	    srcLock->exclusiveLock();
	    dstLock->exclusiveLock();
	    srcAcc = (Account*)srcShm->get();
	    dstAcc = (Account*)dstShm->get();
	    srcAcc->amount -= 1;
	    dstAcc->amount += 1;
	    srcShm->set(srcAcc, sizeof(srcAcc));
	    dstShm->set(dstAcc, sizeof(srcAcc));
	    dstLock->unlock();
	    srcLock->unlock();
	}
	barrier->waitFor();
	srcAcc = (Account*)srcShm->get();
	dstAcc = (Account*)dstShm->get();
	assert(srcAcc->amount == 0);
	assert(dstAcc->amount == initialAmount);
	srcShm->close();
	dstShm->close();
	srcLock->close();
	dstLock->close();
	barrier->close();
    }
};



int main(int argc, char* argv[]) { 
    Bank test;
    return test.main(argc ,argv);
}


