#include "RIPCTest.h"

/**
 * Test for deadlock detection
 * Usage:
 * Start N processes with "Deadlock [-nodeadlock] [-timed] N" command
 * Output: "Number of detected deadlocks: XXX"
 */
class Deadlock : public RIPCTest { 
  public:
    void run(RIPCSession* session, int id) { 
	int nParticipants = nProcesses;
	int i;
	RIPCBarrier* barrier = session->createBarrier("barrier", nParticipants);
	RIPCLock** lock = new RIPCLock*[nParticipants];
	for (i = 0; i < nParticipants; i++) { 
	    char name[64];
	    sprintf(name, "lock%d", i);
	    lock[i] = session->createLock(name);
	}
	int nDeadlocks = 0;

	barrier->waitFor();
	for (i = 0; i < nIterations; i++) { 
	    int firstLock = rand() % nParticipants;
	    int secondLock = rand() % nParticipants;
	    bool firstLockIsExclusive = true;
	    bool secondLockIsExclusive = true;
	    if (options & NO_DEADLOCK) { 
		if (firstLock > secondLock) { 		
		    int tmp = firstLock;
		    firstLock = secondLock;
		    secondLock = tmp;
		}
	    } else { 
		firstLockIsExclusive = rand() & 1;
		secondLockIsExclusive = rand() & 1;
	    }
	    if (options & TIMED_WAIT) { 
		bool succeed;
		if (firstLockIsExclusive) { 
		    lock[firstLock]->exclusiveLock();
		} else {
		    lock[firstLock]->sharedLock();
		}
		if (secondLockIsExclusive) { 
		    succeed = lock[secondLock]->exclusiveLock(timeout);
		} else {
		    succeed = lock[secondLock]->sharedLock(timeout);
		}
		if (succeed) { 
		    if (firstLock != secondLock) { 
			lock[secondLock]->unlock();
		    }
		} else { 
		    nDeadlocks += 1;
		}
	    } else { 
		if (firstLockIsExclusive) { 
		    lock[firstLock]->exclusiveLock();
		} else {
		    lock[firstLock]->sharedLock();
		}
		try { 
		    if (secondLockIsExclusive) { 
			lock[secondLock]->exclusiveLock();
		    } else {
			lock[secondLock]->sharedLock();
		    }
		    if (firstLock != secondLock) { 
			lock[secondLock]->unlock();
		    }
		} catch (RIPCDeadlockException const&) { 
		    nDeadlocks += 1;
		}
	    }
	    lock[firstLock]->unlock();
	} 
	printf("Number of detected deadlocks: %d\n", nDeadlocks);
    }
};



int main(int argc, char* argv[]) { 
    Deadlock test;
    return test.main(argc ,argv);
}




