#include "RIPCTest.h"

/**
 * Philosophres at dinner.
 * Test for semaphores.
 * Usage: start N processes by "Philosopher [-nodeadlock] N id" command, where 0 <= id < N
 * Output: "Number of faults: XXX"
 */
class Philosophers : public RIPCTest { 
  public:
    void run(RIPCSession* session, int id) {
	int nPhilosophers = nProcesses;
	int nForks = nPhilosophers;
	int i;
	RIPCSemaphore** fork = new RIPCSemaphore*[nForks];

	for (i = 0; i < nForks; i++) { 
	    char name[15];
	    sprintf(name, "lock%d", i);
	    fork[i] = session->createSemaphore(name, 1);
	}

	RIPCBarrier* barrier = session->createBarrier("philosophers", nPhilosophers);
	barrier->waitFor();

	int nFaults = 0;
	for (i = 0; i < nIterations; i++) { 
	    for (int j = 0; j < nPhilosophers; j++) { 
		int leftFork = j == 0 ? nForks-1 : j-1;
		int rightFork = j;
		int takeFirst, takeSecond;
		if (options & NO_DEADLOCK) { 
		    if (leftFork > rightFork) { 
			takeFirst = rightFork;
			takeSecond = leftFork;
		    } else { 
			takeFirst = leftFork;
			takeSecond = rightFork;
		    }
		} else {
		    if (rand() & 1) { 
			takeFirst = leftFork;
			takeSecond = rightFork;
		    } else { 
			takeFirst = rightFork;
			takeSecond = leftFork;
		    }
		}
		fork[takeFirst]->waitFor();
		if (!fork[takeSecond]->waitFor(timeout)) { 
		    nFaults += 1;
		} else { 
		    fork[takeSecond]->signal();
		}
		fork[takeFirst]->signal();
	    }
	}
	printf("Number of faults: %d\n", nFaults);
    }
};



int main(int argc, char* argv[]) { 
    Philosophers test;
    return test.main(argc ,argv);
}


