#ifndef __RIPC_TEST_H__
#define __RIPC_TEST_H__

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <assert.h>

#include "RIPCThread.h"
#include "RIPCFactory.h"
#include "RIPCSession.h"
#include "RIPCServerFactory.h"
#include "RIPCClientFactory.h"
#include "ripc.h"

class RIPCTest { 
  public:
    class RIPCTestThread : public RIPCThread { 
      public:
	void start() { 
	    RIPCThread::start(&runTest, this);
	}
	static void thread_proc runTest(void* arg) { 
	    ((RIPCTestThread*)arg)->run();
	}
	void run() { 
	    test->startSession(id);
	}
	RIPCTestThread(RIPCTest* test, int id) { 
	    this->test = test;
	    this->id = id;
	}
	int id;
	RIPCTest* test;
    };

    virtual void run(RIPCSession* session, int id) = 0;

    void startSession(int id) { 
	RIPCSession* session = getSession();
	run(session, id);
	delete session;
    }

    RIPCSession* getSession() { 
	return factory->create(hostname, port);
    }

    int main(int argc, char* argv[]) 
    { 
	int i;
	int nParams = 0;
	int param;
	int id = 0;
	for (i = 1; i < argc; i++) { 
	    if (*argv[i] == '-') { 
		if (strcmp(argv[i], "-port") == 0) { 
		    if (i+1 == argc || sscanf(argv[i+1], "%d", &port) != 1) { 
			fprintf(stderr, "Invalid port number\n");
			return 1;
		    }
		    i += 1;
		} else if (strcmp(argv[i], "-iterations") == 0) { 
		    if (i+1 == argc || sscanf(argv[i+1], "%d", &nIterations) != 1) { 
			fprintf(stderr, "Invalid number of iterations\n");
			return 1;
		    }
		    i += 1;
		} else if (strcmp(argv[i], "-host") == 0) { 
		    if (i+1 == argc) { 
			fprintf(stderr, "Hostname is missing\n");
			return 1;
		    }
		    hostname = argv[++i];
		} else if (strcmp(argv[i], "-nodeadlock") == 0) { 
		    options |= NO_DEADLOCK;
		} else if (strcmp(argv[i], "-timed") == 0) { 
		    options |= TIMED_WAIT;
		} else if (strcmp(argv[i], "-timeout") == 0) { 
		    if (i+1 == argc || sscanf(argv[i+1], "%d", &timeout) != 1) { 
			fprintf(stderr, "Invalid value of timeout\n");
			return 1;
		    }
		    i += 1;
		} else { 
		    fprintf(stderr, "Unknown option %s\n", argv[i]);
		    return 1;
		}
	    } else { 
		if (sscanf(argv[i], "%d", &param) != 1) { 
		    fprintf(stderr, "Invalid parameter (integer number excepted)\n");
		    return 1;
		}
		if (nParams == 0) { 
		    nProcesses = param;
		} else if (nParams == 1) { 
		    id = param;
		} else { 
		    fprintf(stderr, "Test should be called with one or two parameters: N_PROCESSES [ID]\n");
		    return 1;
		}
		nParams += 1;
	    }	    		
	}
	if (port == 0) { 
	    factory = RIPCServerFactory::getInstance();
	} else { 
	    factory = RIPCClientFactory::getInstance();
	}
	time_t started = time(NULL);
	if (nParams == 2) { 
	    startSession(id);
	} else { 
	    RIPCTestThread** tests = new RIPCTestThread*[nProcesses];
	    for (i = 0; i < nProcesses; i++) { 
		tests[i] = new RIPCTestThread(this, i);
		tests[i]->start();
	    }
	    for (i = 0; i < nProcesses; i++) { 
		tests[i]->join();
	    }
	    printf("Elapsed time %u\n", (unsigned)(time(NULL) - started));
	}
	return 0;
    }
    
    RIPCTest() 
    {
	port = 6000;
	hostname = "localhost";
	nProcesses = 5;
	options = 0;
	nIterations = 1000;
	timeout = 100;
    }

    enum Options { 
	NO_DEADLOCK = 1,
	TIMED_WAIT  = 2
    };

    int          port;
    int          nProcesses;
    char const*  hostname;
    int          options;
    int          nIterations;
    unsigned     timeout;
    RIPCFactory* factory;
};

#endif

 

